#
# Copyright 2017 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Package "web"
"""
# pylint: disable=W0403,W0611,W0622
API_VERSION = "1.5.3"

import action
import database
import host
import jetstream
import snapshot
import group
import sourceconfig
import service
import about
import notification
import permission
import user
import alert
import environment
import policy
import session
import replication
import capacity
import data
import domain
import connectivity
import network
import timeflow
import storage
import fault
import namespace
import system
import repository
import delphix
import source
import authorization
import analytics
import registration
import job
import role
import toolkit
