#
# Copyright 2017 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Package "jetstream.usage.template"
"""
# pylint: disable=W0403,W0611,W0622
API_VERSION = "1.6.0"

import urllib
from delphixpy.v1_6_0 import response_validator

def list_template_usage_data(engine, template=None):
    """
    Lists the usage breakdown for data templates.

    :param engine: The Delphix Engine
    :type engine: :py:class:`delphixpy.v1_6_0.delphix_engine.DelphixEngine`
    :param template: If passed in, this query parameter restricts the API to
        only return the usage information for the given data template. By
        default, the usage information for all data templates is returned.
    :type template: ``basestring``
    :rtype: ``list`` of :py:class:`v1_6_0.web.vo.JSTemplateUsageData`
    """
    assert API_VERSION == engine.API_VERSION, "Wrong API version (%s) for parameter 'engine' (%s)" % (API_VERSION, engine.API_VERSION)
    url = "/resources/json/delphix/jetstream/usage/template/listTemplateUsageData"
    query_params = {"template": template}
    query_dct = {k: query_params[k] for k in query_params if query_params[k] is not None}
    if query_dct:
        url_params = urllib.urlencode(query_dct)
        url += "?%s" % url_params
    response = engine.get(url)
    result = response_validator.validate(response, engine)
    raw_result = getattr(engine, 'raw_result', False)
    return response_validator.parse_result(result, undef_enabled=True, return_types=[u'JSTemplateUsageData'], returns_list=True, raw_result=raw_result)

